dojo.require("dojox.drawing.defaults");

var drawingDefaults = dojox.drawing.defaults;
drawingDefaults.clickMode = false;

var w;
var dialog;
var savedData = "";
var autoSaveTimeout; 


function buttonMouseEnter(obj) {
  dojo.addClass(obj, "hover");
};

function buttonMouseLeave(obj) {
  dojo.removeClass(obj, "hover");
};

function changeColour(){
  window.parent.drawingOptionsDialog(drawingObject);
};

function buttonClick(obj, func, autoSelect) {
  if (!autoSelect)
    dojo.addClass(obj, "selected");
  setTimeout(function() {
    if (func) func();
    if (!autoSelect)
      dojo.removeClass(obj, "selected");
  }, 200);
};

function parseScore(old, text){
  var s = text || "";
  s = s.split(" ").join("");
  s = s.split(",").join(".");
  var v = parseFloat(s);
  if (v && (v == s)) {
    if (old)
       return old + v;
    else
       return v;    
  }
  else  {
    if (old)
       return old;
     else
       return null;
  }
};


function handleError(err) {
  var msg;
  if (dojo.isObject(err)) msg = err.message || err.toString();
  else msg = err;
  msg = msg || "Unknown error";
  alert(msg);
};

function transForm(obj, propname, oldWidth, newWidth){
  if (obj[propname] && oldWidth) 
     obj[propname] = obj[propname] * newWidth / oldWidth;
};

var params = {};


function saveMarks() {

  if (params["readonly"] == "1")
    return;

  if (autoSaveTimeout) {
    clearTimeout(autoSaveTimeout);
    autoSaveTimeout = null;
  };

  var score = null;
  var o = [];
  for (var id in drawingObject.stencils.stencils) {
    var stencil = drawingObject.stencils.stencils[id];

    //console.log(stencil);

    var data = stencil.exporter();
	if(data.type=="path"){ 
       if (stencil.closePath === false)
         data.data = {closePath: false};
         if (data.style) {
           data["Style"] = data.style;
           delete data.style;  
         }
    } 
    else if (data.type == "textBlock") {
      score = parseScore(score, stencil._rawText);
    };
    if (data.type!="image")
      o.push(data);
  }

  if (o.length)
    o.push({
      type: "globals",
      width: w
    });

  var exported = o.length?dojo.toJson(o, true):"";

  if (exported == savedData) {
    if (autoSaveTimeout) clearTimeout(autoSaveTimeout);
    autoSaveTimeout = setTimeout(saveMarks, 10 * 1000);
    return;
  }

  var form = dojo.byId("workimageeditform");
  form.elements["id"].value = params.id;
  form.elements["marks"].value = exported;
  form.elements["score"].value = score;

  dojo.io.iframe.send({

    method: "POST",
    form: form,
    handleAs: "html",

    handle: function(htmlResponse, ioArgs) {
      var response = dojo.fromJson(htmlResponse.body.textContent || htmlResponse.body.innerText);
      console.log("handle", response, ioArgs)
      if (!response.OK) handleError(response)
      else   {
        savedData = exported;
        if (window.parent.scoreChanged) window.parent.scoreChanged(response.work, response.score);

        if (autoSaveTimeout) clearTimeout(autoSaveTimeout);
        autoSaveTimeout = setTimeout(saveMarks, 10 * 1000);
      }
    }
  });
};

var transparentView = false;

function toggleMarks(){
 
   transparentView = !transparentView;

   for (var id in drawingObject.stencils.stencils) {
     var stencil = drawingObject.stencils.stencils[id];
     if (stencil.shortType != "image")
       stencil.container.rawNode.style.opacity = transparentView?0.2:1;
    }

    if (transparentView) 
      dojo.addClass(dojo.byId("toggleMarksButton"), "selected");
    else  
      dojo.removeClass(dojo.byId("toggleMarksButton"), "selected")
};

dojo.addOnLoad(function() {


  dojo.registerModulePath("redpen", "../../redpen");

  dojox.drawing.tools.Arrow.setup.tooltip = "Fletxa";
  dojox.drawing.tools.Ellipse.setup.tooltip = "El·lipse";
  dojox.drawing.tools.Line.setup.tooltip = "Línia";
  dojox.drawing.tools.Path.setup.tooltip = "Polígon";
  dojox.drawing.tools.Pencil.setup.tooltip = "Llapis";
  dojox.drawing.tools.Rect.setup.tooltip = "Rectangle";
  dojox.drawing.tools.TextBlock.setup.tooltip = "Text";


  params = {};
  var href = document.location.href;
  href = href.substr(href.indexOf("?") + 1);
  dojo.forEach(href.split("&"), function(pair) {
    var key = pair.split("=");
    var value = key[1];
    key = key[0];
    params[key] = value;
  });

  if (!params.id) {
    alert("Work not assigned.");
    return;
  }

  if (!params.width || !params.height) {
    alert("Dimensions not assigned.");
    return;
  }


  if (params["readonly"] == "1")   {
    var node = dojo.byId("drawingToolBar");
    console.log("node", node);
    node.parentNode.removeChild(node);
  }
  else {
    var node = dojo.byId("drawingToolBarRO");
    console.log("node", node);
    node.parentNode.removeChild(node);
  };

  dojo.parser.parse();

  dojo.connect(drawingObject, "set" , function(name, value){
     if ((name == "focused") && value) {
        if (window.parent.setFocusFrame) window.parent.setFocusFrame(params.id);
        dijit.focus(drawingObject.domNode);
     }
  });

  dojo.connect(drawingObject.mouse, "onDown", function(){
     if (window.parent.setFocusFrame) window.parent.setFocusFrame(params.id);
  });


  dojo.connect(drawingObject.canvas, "setZoom", function(){
     if (window.parent.setZoomFrame) window.parent.setZoomFrame(params.id, drawingObject.canvas.zoom);
  });


  var widthPercent = 0.55;

  w = dojo.coords(document.body).w;

  dojo.byId("wrapper").style.height = Math.round(params.height * Math.round(w * widthPercent) / params.width);

  dojo.connect(drawingObject, "onSurfaceReady", function() {
    var img = drawingObject.addStencil("image", {
      data: {
        x: Math.round(w * widthPercent / 2),
        y: 0,
        width: Math.round(w * widthPercent),
        height: Math.round(params.height * Math.round(w * widthPercent) / params.width),
        src: "/redpendata/workimage.jpeg?id=" + params.id
      }
    });
    img.container.rawNode.style.pointerEvents = "none";

    drawingObject.changeDefaults({
      norm: {
        fill: "transparent",
        width: 5,
        color: "red"
      },
      text: {
        color: "red"
      }
    });

    dojo.xhrGet({
      url: "/redpendata/workimagemarks.json",
      handleAs: "json",
      content: {
        id: params.id
      },
      load: function(response) {
        if (!response.OK) {
          handleError(response);
        }
        else {
          setTimeout(function() {
            console.log(dojo.fromJson(response.data));

            var data = dojo.fromJson(response.data);
            if (data) {
              var globals = data.pop();
              dojo.forEach(data, function(stencil){
                 dojo.forEach(["cx", "cy", "rx", "ry", "x1", "y1", "x2", "y2", "x", "y", "width", "height"], function(propname){
                    transForm(stencil, propname, globals.width, w);              
                 });
                 dojo.forEach(stencil.points || [], function(point){
                    transForm(point, "x", globals.width, w);              
                    transForm(point, "y", globals.width, w);              
                 });
              });

	          dojo.forEach(data, function(m){
                try {
                  var st = drawingObject.addStencil(m.type, m);
                  if (st.shortType == "path")  {
                       st.style.current.color = m.color;
                       st.style.current.fill = m.fill;
                       st.render();   
                  };

                } catch(e) {
                  console.log("Stencil load error", m, e);
                };
              }, drawingObject);

              savedData = data;
            } 

            if (params["readonly"] == "1")   {

              for (var id in drawingObject.stencils.stencils) {
                var stencil = drawingObject.stencils.stencils[id];
                stencil.container.rawNode.style.pointerEvents = "none";
              }

            }
            else {
              if (autoSaveTimeout) clearTimeout(autoSaveTimeout);
              autoSaveTimeout = setTimeout(saveMarks, 10 * 1000);
            }

          }, 100);
        }
      },
      error: handleError
    });

  });

});

dojox.drawing.tools.Pencil.setup.iconClass = "redpenbutton iconPencil";
dojox.drawing.tools.Path.setup.iconClass = "redpenbutton iconPath";
dojo.extend(dojox.drawing.tools.Pencil, {
  checkClosePoint: function(){
    return false;
  }
});




